searchData={"items":[{"type":"module","title":"ExMemvid","doc":"# ExMemvid\n\nAn Elixir Port of [Memvid](https://github.com/Olow304/memvid).\n\nExMemvid is a proof-of-concept library for storing and retrieving large amounts of text data by encoding it into a video file composed of QR code frames. It leverages modern Elixir libraries for machine learning, video processing, and vector search to provide a unique solution for data storage and semantic retrieval.","ref":"ExMemvid.html"},{"type":"module","title":"How it Works - ExMemvid","doc":"The core idea is to treat video frames as a data storage medium. Each frame in the video contains a QR code that holds a chunk of text. A separate search index is created using text embeddings to allow for fast, semantic searching of the content stored in the video.","ref":"ExMemvid.html#module-how-it-works"},{"type":"module","title":"Encoding Process - ExMemvid","doc":"1.  **Text Chunking**: The input text is divided into smaller, manageable chunks.\n2.  **Embedding**: A sentence transformer model from Hugging Face (via `Bumblebee`) generates a vector embedding for each text chunk.\n3.  **QR Code Generation**: Each text chunk is serialized (optionally with Gzip compression) and encoded into a QR code image.\n4.  **Video Encoding**: The QR code images are compiled into a video file, where each image becomes a single frame. The library uses `Xav` and `Evision` (OpenCV bindings) for this.\n5.  **Index Creation**: The vector embeddings are stored in an HNSWLib (Hierarchical Navigable Small World) index for efficient similarity search. This index maps the embeddings to their corresponding frame numbers in the video.\n6.  **Saving**: The final video file and the search index are saved to disk.","ref":"ExMemvid.html#module-encoding-process"},{"type":"module","title":"Retrieval Process - ExMemvid","doc":"1.  **Search Query**: The user provides a text query.\n2.  **Query Embedding**: The query is converted into a vector embedding using the same model as the encoding process.\n3.  **Semantic Search**: The HNSWLib index is queried to find the text chunks with embeddings most similar to the query's embedding.\n4.  **Frame Identification**: The search results from the index provide the frame numbers where the relevant text chunks are stored.\n5.  **Frame Decoding**: The `Retriever` seeks to the specific frames in the video file, reads the QR codes, and decodes them to retrieve the original text chunks.\n6.  **Result Aggregation**: The retrieved text chunks are returned to the user.","ref":"ExMemvid.html#module-retrieval-process"},{"type":"module","title":"Features - ExMemvid","doc":"*   **Data Archiving**: Store large text corpora in a compressed video format.\n*   **Semantic Search**: Go beyond keyword matching with state-of-the-art text embeddings.\n*   **Configurable**: Easily configure everything from the video codec and QR code version to the embedding model.\n*   **Concurrent**: Utilizes Elixir's concurrency to parallelize embedding and frame decoding tasks.\n*   **Extensible**: The `Embedding` behaviour allows for swapping out the embedding implementation.","ref":"ExMemvid.html#module-features"},{"type":"module","title":"Installation - ExMemvid","doc":"Add `ex_memvid` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_memvid, github: \"edlontech/ex_memvid\", branch: \"main\"}\n  ]\nend\n```\n\nYou will also need `ffmpeg` installed on your system for some of the underlying video operations.","ref":"ExMemvid.html#module-installation"},{"type":"module","title":"Usage Example - ExMemvid","doc":"Here’s a complete example of how to encode a list of text chunks and then retrieve them via search.\n\n```elixir\n# 1. Configuration\nconfig = ExMemvid.Config.validate!(\n  embedding: %{\n    # Using a mock embedding module for this example\n    module: ExMemvid.MockEmbedding,\n    dimension: 3\n  }\n)\n\n# Some text data to store\nchunks = [\n  \"elixir is a functional, concurrent language\",\n  \"phoenix is a popular web framework for elixir\",\n  \"the actor model enables robust, concurrent systems\",\n  \"bumblebee brings hugging face transformers to elixir\"\n]\n\n# 2. Setup Encoder\n{:ok, encoder} = ExMemvid.Encoder.new(config)\nencoder = ExMemvid.Encoder.add_chunks(encoder, chunks)\n\n# 3. Define output paths\noutput_dir = \"output\"\nvideo_path = Path.join(output_dir, \"my_video.mkv\")\nindex_path = Path.join(output_dir, \"my_index.json\")\n\n# 4. Build the video and search index\n{:ok, stats} = ExMemvid.Encoder.build_video(encoder, video_path, index_path)\nIO.inspect(stats)\n\n# 5. Start the Retriever GenServer\n{:ok, retriever_pid} = ExMemvid.Retriever.start_link(\n  video_path: video_path,\n  index_path: index_path,\n  config: config\n)\n\n# 6. Perform a search\nquery = \"What is elixir?\"\n{:ok, results} = ExMemvid.Retriever.search(retriever_pid, query, 2)\n\nIO.puts(\"Search results for '#{query}':\")\nIO.inspect(results)\n#=> [\"elixir is a functional, concurrent language\", \"bumblebee brings hugging face transformers to elixir\"]\n```","ref":"ExMemvid.html#module-usage-example"},{"type":"module","title":"ExMemvid.Config","doc":"Configuration defaults and constants for ExMemvid.\n\nThis module provides a Nimble Options schema for validating and managing\nExMemvid configuration options, converted from the Python version.","ref":"ExMemvid.Config.html"},{"type":"function","title":"ExMemvid.Config.codec_parameters/0","doc":"Returns codec parameters for the specified codec.","ref":"ExMemvid.Config.html#codec_parameters/0"},{"type":"function","title":"ExMemvid.Config.get_codec_parameters/1","doc":"Get codec parameters for a specific codec.","ref":"ExMemvid.Config.html#get_codec_parameters/1"},{"type":"function","title":"Examples - ExMemvid.Config.get_codec_parameters/1","doc":"iex> ExMemvid.Config.get_codec_parameters(:h265)\n    %{video_fps: 30, ...}\n    \n    iex> ExMemvid.Config.get_codec_parameters(:invalid)\n    {:error, \"Unsupported codec: invalid. Available: [:h265, :hevc, :h264]\"}","ref":"ExMemvid.Config.html#get_codec_parameters/1-examples"},{"type":"type","title":"ExMemvid.Config.t/0","doc":"* `:qr` (`t:map/0`) - QR Code generation settings The default value is `%{gzip: false, error_correction: :medium, fill_color: \"#000000\", back_color: \"#ffffff\"}`.\n\n  * `:error_correction` (`t:atom/0`) - Error correction level The default value is `{:in, [:low, :medium, :quartile, :high]}`.\n\n  * `:fill_color` (`t:String.t/0`) - QR fill color The default value is `\"#000000\"`.\n\n  * `:back_color` (`t:String.t/0`) - QR background color The default value is `\"#ffffff\"`.\n\n  * `:gzip` (`t:boolean/0`) - Whether to gzip the QR code text The default value is `false`.\n\n* `:codec` (`t:atom/0`) - Video codec to use: :h265, :hevc, :h264 The default value is `:h265`.\n\n* `:chunking` (`t:map/0`) - Text chunking settings The default value is `%{chunk_size: 1024, overlap: 32}`.\n\n  * `:chunk_size` (`t:integer/0`) - Default chunk size for processing The default value is `1024`.\n\n  * `:overlap` (`t:integer/0`) - Overlap between chunks The default value is `32`.\n\n* `:retrieval` (`t:map/0`) - Data retrieval settings\n\n  * `:top_k` (`t:integer/0`) - Number of top results to retrieve The default value is `5`.\n\n  * `:batch_size` (`t:integer/0`) - Batch size for processing The default value is `100`.\n\n  * `:max_workers` (`t:integer/0`) - Maximum number of worker processes The default value is `4`.\n\n  * `:cache_size` (`t:integer/0`) - Number of frames to cache The default value is `1000`.\n\n* `:embedding` (`t:map/0`) - Text embedding settings The default value is `%{module: ExMemvid.Embedding.Local, batch_size: 32, model: \"sentence-transformers/all-MiniLM-L6-v2\", dimension: 384, max_sequence_length: 512, partitions: 4}`.\n\n  * `:module` (`t:atom/0`) - Which Embedding Implementation to use The default value is `ExMemvid.Embedding.Local`.\n\n  * `:model` (`t:String.t/0`) - Embedding model name The default value is `\"sentence-transformers/all-MiniLM-L6-v2 \"`.\n\n  * `:dimension` (`t:integer/0`) - Embedding vector dimension The default value is `384`.\n\n  * `:batch_size` (`t:integer/0`) - Batch size for embedding The default value is `32`.\n\n  * `:max_sequence_length` (`t:integer/0`) - Maximum sequence length for text input The default value is `512`.\n\n  * `:partitions` (`t:integer/0`) - Number of embedding worker partitions (defaults to CPU cores) The default value is `4`.\n\n* `:index` (`t:map/0`) - Vector index settings The default value is `%{vector_search_space: :cosine, embedding_dimensions: 384, max_elements: 10000}`.\n\n  * `:ef_construction` (`t:integer/0`) - Number of elements to consider during index construction The default value is `400`.\n\n  * `:ef_search` (`t:integer/0`) - Number of elements to consider during search The default value is `50`.\n\n  * `:vector_search_space` - Vector search space type The default value is `:cosine`.\n\n  * `:embedding_dimensions` (`t:integer/0`) - Dimensions of the embedding vectors The default value is `384`.\n\n  * `:max_elements` (`t:integer/0`) - Maximum number of elements in the index The default value is `10000`.","ref":"ExMemvid.Config.html#t:t/0"},{"type":"function","title":"ExMemvid.Config.validate!/1","doc":"Validates configuration options, raising on error.","ref":"ExMemvid.Config.html#validate!/1"},{"type":"behaviour","title":"ExMemvid.Embedding","doc":"Behavior module for generating text embeddings used in semantic search and retrieval.\n\nThis module defines the interface for converting text into high-dimensional vector\nrepresentations (embeddings) that can be used for similarity search, clustering,\nand other machine learning tasks within the ExMemvid system.","ref":"ExMemvid.Embedding.html"},{"type":"behaviour","title":"Implementation - ExMemvid.Embedding","doc":"Modules implementing this behavior must provide:\n- `embed_text/2` - Generate embedding for a single text string\n- `embed_texts/2` - Generate embeddings for multiple text strings (batch processing)","ref":"ExMemvid.Embedding.html#module-implementation"},{"type":"behaviour","title":"Embedding Format - ExMemvid.Embedding","doc":"All embeddings are returned as `Nx.Tensor` structures with:\n- Data type: `f32` (32-bit floating point)\n- Single text: 1D tensor with shape `[embedding_dim]`\n- Multiple texts: 2D tensor with shape `[batch_size, embedding_dim]`\n- Typical dimensions: 384, 512, 768, or 1024 depending on the model","ref":"ExMemvid.Embedding.html#module-embedding-format"},{"type":"behaviour","title":"Example Implementation - ExMemvid.Embedding","doc":"defmodule MyEmbeddingProvider do\n      @behaviour ExMemvid.Embedding\n\n      @impl true\n      def embed_text(text, opts) do\n        # Your embedding logic here\n        {:ok, embedding_tensor}\n      end\n\n      @impl true\n      def embed_texts(texts, opts) do\n        # Batch embedding logic here\n        {:ok, batch_embeddings_tensor}\n      end\n    end","ref":"ExMemvid.Embedding.html#module-example-implementation"},{"type":"behaviour","title":"Usage - ExMemvid.Embedding","doc":"# Using the convenience functions\n    {:ok, embedding} = ExMemvid.Embedding.embed_text(MyProvider, \"Hello world\", %{})\n    {:ok, embeddings} = ExMemvid.Embedding.embed_texts(MyProvider, [\"Hello\", \"world\"], %{})\n\n    # Direct implementation call\n    {:ok, embedding} = MyProvider.embed_text(\"Hello world\", %{})","ref":"ExMemvid.Embedding.html#module-usage"},{"type":"behaviour","title":"Error Handling - ExMemvid.Embedding","doc":"Implementations should return `{:error, reason}` for various failure cases:\n- `:empty_text` - When provided text is empty or whitespace-only\n- `:model_not_loaded` - When the embedding model is unavailable\n- `:invalid_input` - When text contains unsupported characters or format\n- `:timeout` - When embedding generation exceeds time limits","ref":"ExMemvid.Embedding.html#module-error-handling"},{"type":"callback","title":"ExMemvid.Embedding.embed_text/2","doc":"Generates embeddings for a single text string.","ref":"ExMemvid.Embedding.html#c:embed_text/2"},{"type":"callback","title":"Examples - ExMemvid.Embedding.embed_text/2","doc":"iex> ExMemvid.Embedding.embed_text(pid, \"Hello world\")\n    {:ok, #Nx.Tensor }\n    \n    iex> ExMemvid.Embedding.embed_text(pid, \"\")\n    {:error, :empty_text}","ref":"ExMemvid.Embedding.html#c:embed_text/2-examples"},{"type":"function","title":"ExMemvid.Embedding.embed_text/3","doc":"","ref":"ExMemvid.Embedding.html#embed_text/3"},{"type":"callback","title":"ExMemvid.Embedding.embed_texts/2","doc":"Generates embeddings for a list of text strings.","ref":"ExMemvid.Embedding.html#c:embed_texts/2"},{"type":"callback","title":"Examples - ExMemvid.Embedding.embed_texts/2","doc":"iex> ExMemvid.Embedding.embed_texts(pid, [\"Hello\", \"world\"])\n    {:ok, #Nx.Tensor }","ref":"ExMemvid.Embedding.html#c:embed_texts/2-examples"},{"type":"function","title":"ExMemvid.Embedding.embed_texts/3","doc":"","ref":"ExMemvid.Embedding.html#embed_texts/3"},{"type":"type","title":"ExMemvid.Embedding.embedding/0","doc":"","ref":"ExMemvid.Embedding.html#t:embedding/0"},{"type":"module","title":"ExMemvid.Embedding.Local","doc":"Text embedding module using Bumblebee for semantic search.\n\nThis module acts as a facade for the partitioned embedding workers.\nIt uses round-robin selection to distribute embedding requests across\nmultiple worker processes for better performance and load distribution.","ref":"ExMemvid.Embedding.Local.html"},{"type":"function","title":"ExMemvid.Embedding.Local.get_worker/0","doc":"Gets a specific worker for direct access (mainly for testing).","ref":"ExMemvid.Embedding.Local.html#get_worker/0"},{"type":"function","title":"ExMemvid.Embedding.Local.start_link/1","doc":"Starts the embedding supervisor with the given configuration.","ref":"ExMemvid.Embedding.Local.html#start_link/1"},{"type":"module","title":"ExMemvid.Embedding.Local.Worker","doc":"Individual embedding worker process.\n\nThis GenServer represents a single embedding worker that handles\ntext embedding requests using Bumblebee. Multiple workers are\nmanaged by the PartitionSupervisor for load distribution.","ref":"ExMemvid.Embedding.Local.Worker.html"},{"type":"function","title":"ExMemvid.Embedding.Local.Worker.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExMemvid.Embedding.Local.Worker.html#child_spec/1"},{"type":"function","title":"ExMemvid.Embedding.Local.Worker.embed_text/3","doc":"","ref":"ExMemvid.Embedding.Local.Worker.html#embed_text/3"},{"type":"function","title":"ExMemvid.Embedding.Local.Worker.embed_texts/3","doc":"","ref":"ExMemvid.Embedding.Local.Worker.html#embed_texts/3"},{"type":"function","title":"ExMemvid.Embedding.Local.Worker.start_link/1","doc":"","ref":"ExMemvid.Embedding.Local.Worker.html#start_link/1"},{"type":"module","title":"ExMemvid.Embedding.Supervisor","doc":"Supervisor for managing partitioned embedding workers.\n\nThis supervisor manages multiple instances of embedding workers using PartitionSupervisor\nfor better performance and load distribution. Workers are selected using round-robin\nto distribute the embedding workload across available CPU cores.","ref":"ExMemvid.Embedding.Supervisor.html"},{"type":"function","title":"ExMemvid.Embedding.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExMemvid.Embedding.Supervisor.html#child_spec/1"},{"type":"function","title":"ExMemvid.Embedding.Supervisor.get_worker/0","doc":"Gets the next available worker using round-robin selection.","ref":"ExMemvid.Embedding.Supervisor.html#get_worker/0"},{"type":"function","title":"ExMemvid.Embedding.Supervisor.get_worker_for_partition/1","doc":"Gets worker for a specific partition (used by PartitionSupervisor).","ref":"ExMemvid.Embedding.Supervisor.html#get_worker_for_partition/1"},{"type":"function","title":"ExMemvid.Embedding.Supervisor.start_link/1","doc":"","ref":"ExMemvid.Embedding.Supervisor.html#start_link/1"},{"type":"module","title":"ExMemvid.Encoder","doc":"A Encoder implementation for encoding text chunks into QR code videos.\n\nEach encoder instance is responsible for managing one video encoding process\nwith proper state transitions and error handling.","ref":"ExMemvid.Encoder.html"},{"type":"module","title":"States - ExMemvid.Encoder","doc":"- `:ready` - Ready to accept chunks and configuration\n- `:collecting` - Collecting text chunks for encoding\n- `:building` - Building the video and index files\n- `:completed` - Successfully completed encoding\n- `:error` - Error state with failure information","ref":"ExMemvid.Encoder.html#module-states"},{"type":"module","title":"State Data - ExMemvid.Encoder","doc":"The state machine maintains:\n- `config`: ExMemvid configuration\n- `chunks`: List of text chunks to encode\n- `index_name`: Unique identifier for this encoding process\n- `output_path`: Video output file path\n- `index_path`: Index output file path\n- `stats`: Encoding statistics (when completed)\n- `error`: Error information (when in error state)","ref":"ExMemvid.Encoder.html#module-state-data"},{"type":"module","title":"Example Usage - ExMemvid.Encoder","doc":"# Start a new encoder process\n    {:ok, pid} = ExMemvid.EncoderStateMachine.start_link(\n      index_name: \"my_video_001\",\n      config: %ExMemvid.Config{codec: :h264}\n    )\n\n    # Add text chunks\n    :ok = ExMemvid.EncoderStateMachine.add_chunks(pid, [\"chunk1\", \"chunk2\"])\n\n    # Build the video\n    {:ok, stats} = ExMemvid.EncoderStateMachine.build_video(\n      pid, \n      \"output/video.mp4\", \n      \"output/index.bin\"\n    )","ref":"ExMemvid.Encoder.html#module-example-usage"},{"type":"function","title":"ExMemvid.Encoder.add_chunks/2","doc":"Adds text chunks to the encoder.","ref":"ExMemvid.Encoder.html#add_chunks/2"},{"type":"function","title":"ExMemvid.Encoder.add_text/2","doc":"Adds text and automatically chunks it using the configured chunking settings.","ref":"ExMemvid.Encoder.html#add_text/2"},{"type":"function","title":"ExMemvid.Encoder.build_video/3","doc":"Builds the QR code video and search index.\nThis is an asynchronous operation that transitions the state machine to building state.","ref":"ExMemvid.Encoder.html#build_video/3"},{"type":"function","title":"ExMemvid.Encoder.building/3","doc":"","ref":"ExMemvid.Encoder.html#building/3"},{"type":"function","title":"ExMemvid.Encoder.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExMemvid.Encoder.html#child_spec/1"},{"type":"function","title":"ExMemvid.Encoder.collecting/3","doc":"","ref":"ExMemvid.Encoder.html#collecting/3"},{"type":"function","title":"ExMemvid.Encoder.completed/3","doc":"","ref":"ExMemvid.Encoder.html#completed/3"},{"type":"function","title":"ExMemvid.Encoder.error/3","doc":"","ref":"ExMemvid.Encoder.html#error/3"},{"type":"function","title":"ExMemvid.Encoder.get_state/1","doc":"Gets the current state and data of the encoder.","ref":"ExMemvid.Encoder.html#get_state/1"},{"type":"function","title":"ExMemvid.Encoder.ready/3","doc":"","ref":"ExMemvid.Encoder.html#ready/3"},{"type":"function","title":"ExMemvid.Encoder.reset/1","doc":"Resets the encoder to ready state, clearing all chunks and previous results.","ref":"ExMemvid.Encoder.html#reset/1"},{"type":"function","title":"ExMemvid.Encoder.start_link/1","doc":"Starts a new encoder","ref":"ExMemvid.Encoder.html#start_link/1"},{"type":"function","title":"Options - ExMemvid.Encoder.start_link/1","doc":"- `:index_name` - Unique identifier for this encoder instance (required)\n- `:config` - ExMemvid configuration (required)\n- `:name` - Optional process name for registration","ref":"ExMemvid.Encoder.html#start_link/1-options"},{"type":"type","title":"ExMemvid.Encoder.state_data/0","doc":"","ref":"ExMemvid.Encoder.html#t:state_data/0"},{"type":"type","title":"ExMemvid.Encoder.state_name/0","doc":"","ref":"ExMemvid.Encoder.html#t:state_name/0"},{"type":"module","title":"ExMemvid.EncoderSupervisor","doc":"This supervisor provides a robust way to spawn, monitor, and manage\nindividual encoder state machines. Each encoder process is isolated\nand can fail independently without affecting other encoding operations.","ref":"ExMemvid.EncoderSupervisor.html"},{"type":"module","title":"Features - ExMemvid.EncoderSupervisor","doc":"- **Process Isolation**: Each encoder runs in its own supervised process\n- **Fault Tolerance**: Failed encoders don't affect other running encoders\n- **Dynamic Management**: Start and stop encoders on demand\n- **Resource Cleanup**: Automatic cleanup of failed or completed processes\n- **Process Registry**: Optional registration of encoders by name","ref":"ExMemvid.EncoderSupervisor.html#module-features"},{"type":"module","title":"Usage - ExMemvid.EncoderSupervisor","doc":"# Start an encoder with automatic naming\n    {:ok, pid} = ExMemvid.EncoderSupervisor.start_encoder(\n      index_name: \"document_001\",\n      config: %ExMemvid.Config{codec: :h264}\n    )\n\n    # Start an encoder with custom name\n    {:ok, pid} = ExMemvid.EncoderSupervisor.start_encoder(\n      index_name: \"document_002\", \n      config: config,\n      name: {:via, Registry, {ExMemvid.Registry, \"my_encoder\"}}\n    )\n\n    # Stop a specific encoder\n    :ok = ExMemvid.EncoderSupervisor.stop_encoder(pid)\n\n    # List all running encoders\n    encoders = ExMemvid.EncoderSupervisor.list_encoders()","ref":"ExMemvid.EncoderSupervisor.html#module-usage"},{"type":"function","title":"ExMemvid.EncoderSupervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExMemvid.EncoderSupervisor.html#child_spec/1"},{"type":"function","title":"ExMemvid.EncoderSupervisor.count_encoders/0","doc":"Counts the number of running encoder processes.","ref":"ExMemvid.EncoderSupervisor.html#count_encoders/0"},{"type":"function","title":"ExMemvid.EncoderSupervisor.get_encoders_info/0","doc":"Gets detailed information about all running encoders including their current state.\n\nReturns a map with encoder PIDs as keys and their state information as values.","ref":"ExMemvid.EncoderSupervisor.html#get_encoders_info/0"},{"type":"function","title":"ExMemvid.EncoderSupervisor.list_encoders/0","doc":"Lists all currently running encoder processes.\n\nReturns a list of `{id, pid, type, modules}` tuples for each running encoder.","ref":"ExMemvid.EncoderSupervisor.html#list_encoders/0"},{"type":"function","title":"ExMemvid.EncoderSupervisor.start_encoder/1","doc":"Starts a new encoder state machine under supervision.","ref":"ExMemvid.EncoderSupervisor.html#start_encoder/1"},{"type":"function","title":"Options - ExMemvid.EncoderSupervisor.start_encoder/1","doc":"- `:index_name` - Unique identifier for the encoder (required)\n- `:config` - ExMemvid configuration (required) \n- `:name` - Optional process name for registration\n- `:restart` - Restart strategy (default: `:temporary`)\n- `:shutdown` - Shutdown timeout (default: `5000`)","ref":"ExMemvid.EncoderSupervisor.html#start_encoder/1-options"},{"type":"function","title":"Examples - ExMemvid.EncoderSupervisor.start_encoder/1","doc":"# Basic encoder\n    {:ok, pid} = ExMemvid.EncoderSupervisor.start_encoder(\n      index_name: \"doc_001\",\n      config: %ExMemvid.Config{}\n    )\n\n    # Named encoder with custom restart strategy\n    {:ok, pid} = ExMemvid.EncoderSupervisor.start_encoder(\n      index_name: \"doc_002\",\n      config: config,\n      name: MyEncoder,\n      restart: :transient\n    )","ref":"ExMemvid.EncoderSupervisor.html#start_encoder/1-examples"},{"type":"function","title":"ExMemvid.EncoderSupervisor.start_link/1","doc":"Starts the encoder supervisor.","ref":"ExMemvid.EncoderSupervisor.html#start_link/1"},{"type":"function","title":"ExMemvid.EncoderSupervisor.stop_all_encoders/0","doc":"Stops all running encoder processes.\n\nThis is useful for cleanup during application shutdown or testing.","ref":"ExMemvid.EncoderSupervisor.html#stop_all_encoders/0"},{"type":"function","title":"ExMemvid.EncoderSupervisor.stop_encoder/1","doc":"Stops an encoder process.","ref":"ExMemvid.EncoderSupervisor.html#stop_encoder/1"},{"type":"function","title":"Examples - ExMemvid.EncoderSupervisor.stop_encoder/1","doc":":ok = ExMemvid.EncoderSupervisor.stop_encoder(pid)\n    :ok = ExMemvid.EncoderSupervisor.stop_encoder(MyEncoder)","ref":"ExMemvid.EncoderSupervisor.html#stop_encoder/1-examples"},{"type":"module","title":"ExMemvid.Index","doc":"Manages a high-performance vector similarity search index for encoded video content.\n\nThe `ExMemvid.Index` module provides semantic search capabilities over text chunks\nthat have been encoded into QR code videos. It uses HNSW (Hierarchical Navigable\nSmall World) algorithms for efficient approximate nearest neighbor search, enabling\nfast retrieval of relevant content from large video archives.","ref":"ExMemvid.Index.html"},{"type":"module","title":"Key Features - ExMemvid.Index","doc":"- **Vector Similarity Search**: Uses embeddings to find semantically similar content\n- **Frame Mapping**: Maintains relationships between text chunks and video frames  \n- **Batch Processing**: Efficiently handles multiple text chunks simultaneously\n- **Persistence**: Save and load index state to/from disk\n- **Metadata Management**: Stores searchable metadata alongside vector embeddings\n- **Configurable Performance**: Tunable parameters for speed vs accuracy trade-offs","ref":"ExMemvid.Index.html#module-key-features"},{"type":"module","title":"Architecture - ExMemvid.Index","doc":"The index maintains three key data structures:\n1. **HNSW Vector Index**: High-dimensional similarity search using HNSWLib\n2. **Metadata Store**: Maps chunk IDs to text snippets and frame numbers\n3. **Frame Mapping**: Associates video frames with their constituent text chunks","ref":"ExMemvid.Index.html#module-architecture"},{"type":"module","title":"Workflow - ExMemvid.Index","doc":"1. **Initialize** - Create index with specified dimensions and parameters\n2. **Add Content** - Insert text chunks with their embeddings and frame associations\n3. **Search** - Query for similar content using natural language\n4. **Retrieve** - Get specific chunks by ID or all chunks from a video frame\n5. **Persist** - Save index state for future sessions","ref":"ExMemvid.Index.html#module-workflow"},{"type":"module","title":"Example Usage - ExMemvid.Index","doc":"# Create a new index\n    config = [\n      index: %{\n        vector_search_space: :cosine,\n        embedding_dimensions: 384,\n        max_elements: 10_000,\n        ef_construction: 200,\n        ef_search: 50\n      },\n      embedding: %{\n        module: MyEmbeddingProvider,\n        model: \"all-MiniLM-L6-v2\"\n      }\n    ]\n    {:ok, index} = ExMemvid.Index.new(config)\n\n    # Add text chunks from encoded video\n    chunks = [\"First chunk of text\", \"Second chunk of text\"]\n    frame_numbers = [0, 1]\n    {:ok, index} = ExMemvid.Index.add_items(index, chunks, frame_numbers)\n\n    # Search for similar content\n    {:ok, results} = ExMemvid.Index.search(index, \"find relevant text\", 5)\n\n    # Get all chunks from a specific video frame\n    {:ok, frame_chunks} = ExMemvid.Index.get_chunks_by_frame(index, 0)\n\n    # Save index for later use\n    :ok = ExMemvid.Index.save(index, \"my_index.json\")","ref":"ExMemvid.Index.html#module-example-usage"},{"type":"module","title":"Search Results - ExMemvid.Index","doc":"Search operations return metadata maps containing:\n- `:id` - Unique identifier for the chunk\n- `:text_snippet` - First 100 characters of the original text\n- `:frame_num` - Video frame number containing this chunk","ref":"ExMemvid.Index.html#module-search-results"},{"type":"module","title":"Performance Tuning - ExMemvid.Index","doc":"Key configuration parameters affect performance:\n- `ef_construction` - Higher values improve accuracy during index building (slower)\n- `ef_search` - Higher values improve search accuracy (slower queries)\n- `max_elements` - Maximum number of vectors the index can hold\n- `vector_search_space` - Distance metric (`:cosine`, `:l2`, `:ip`)","ref":"ExMemvid.Index.html#module-performance-tuning"},{"type":"module","title":"Persistence Format - ExMemvid.Index","doc":"The index is saved as two files:\n- `.json` file - Contains metadata and frame mappings\n- `.hnsw` file - Binary HNSW index data","ref":"ExMemvid.Index.html#module-persistence-format"},{"type":"function","title":"ExMemvid.Index.add_items/3","doc":"","ref":"ExMemvid.Index.html#add_items/3"},{"type":"function","title":"ExMemvid.Index.get_chunk_by_id/2","doc":"","ref":"ExMemvid.Index.html#get_chunk_by_id/2"},{"type":"function","title":"ExMemvid.Index.get_chunks_by_frame/2","doc":"","ref":"ExMemvid.Index.html#get_chunks_by_frame/2"},{"type":"function","title":"ExMemvid.Index.get_stats/1","doc":"","ref":"ExMemvid.Index.html#get_stats/1"},{"type":"function","title":"ExMemvid.Index.load/2","doc":"","ref":"ExMemvid.Index.html#load/2"},{"type":"function","title":"ExMemvid.Index.new/1","doc":"","ref":"ExMemvid.Index.html#new/1"},{"type":"function","title":"ExMemvid.Index.save/2","doc":"","ref":"ExMemvid.Index.html#save/2"},{"type":"function","title":"ExMemvid.Index.search/3","doc":"","ref":"ExMemvid.Index.html#search/3"},{"type":"type","title":"ExMemvid.Index.t/0","doc":"","ref":"ExMemvid.Index.html#t:t/0"},{"type":"module","title":"ExMemvid.QR","doc":"Handles QR code generation and decoding for video frame content encoding.\n\nThe `ExMemvid.QR` module provides the core QR code functionality that enables\ntext data to be visually encoded into video frames and later decoded back to\nthe original content. Each QR code represents a chunk of text data that becomes\na single frame in the encoded video.","ref":"ExMemvid.QR.html"},{"type":"module","title":"Configuration - ExMemvid.QR","doc":"QR code behavior is controlled via configuration:\n\n    config = [\n      qr: [\n        back_color: {255, 255, 255},     # White background\n        fill_color: {0, 0, 0},           # Black foreground  \n        error_correction: :medium,        # Error correction level\n        gzip: true                       # Enable compression\n      ]\n    ]","ref":"ExMemvid.QR.html#module-configuration"},{"type":"module","title":"Usage Examples - ExMemvid.QR","doc":"# Configure QR settings\n    config = [\n      qr: [\n        back_color: {255, 255, 255},\n        fill_color: {0, 0, 0}, \n        error_correction: :medium,\n        gzip: true\n      ]\n    ]\n\n    # Encode text data into QR code PNG\n    chunk_data = Jason.encode!(%{id: 1, text: \"Hello world\", frame: 1})\n    {:ok, qr_png_binary} = ExMemvid.QR.encode(chunk_data, config)\n\n    # Decode QR code back to original text\n    {:ok, decoded_text} = ExMemvid.QR.decode(qr_png_binary, config)\n    {:ok, data} = Jason.decode(decoded_text)\n    # data = %{\"id\" => 1, \"text\" => \"Hello world\", \"frame\" => 1}","ref":"ExMemvid.QR.html#module-usage-examples"},{"type":"module","title":"Data Flow - ExMemvid.QR","doc":"1. **Encoding Path**:\n   - Text data → (Optional) Gzip compression → Base64 encoding → QR code → PNG binary\n\n2. **Decoding Path**:\n   - PNG binary → QR detection → Text extraction → (Optional) Base64 decode → Gzip decompress → Original text","ref":"ExMemvid.QR.html#module-data-flow"},{"type":"module","title":"Compression Benefits - ExMemvid.QR","doc":"When `gzip: true` is enabled:\n- Significantly reduces QR code complexity for repetitive text\n- Allows more data per QR code\n- Essential for maximizing storage density in video frames\n- Automatic compression/decompression is transparent to callers","ref":"ExMemvid.QR.html#module-compression-benefits"},{"type":"function","title":"ExMemvid.QR.decode/2","doc":"","ref":"ExMemvid.QR.html#decode/2"},{"type":"function","title":"ExMemvid.QR.encode/2","doc":"","ref":"ExMemvid.QR.html#encode/2"},{"type":"module","title":"ExMemvid.Retriever","doc":"A stateful GenServer for retrieving and decoding data from video memory.\n\nThis module manages the retriever's state, including a cache for decoded\nframes, ensuring that the cache persists across multiple calls. You start\nthe server with `start_link/3` and interact with it via the returned pid.","ref":"ExMemvid.Retriever.html"},{"type":"function","title":"ExMemvid.Retriever.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExMemvid.Retriever.html#child_spec/1"},{"type":"function","title":"ExMemvid.Retriever.search/3","doc":"Performs a semantic search using the running Retriever server.","ref":"ExMemvid.Retriever.html#search/3"},{"type":"function","title":"ExMemvid.Retriever.start_link/1","doc":"Starts the Retriever GenServer.","ref":"ExMemvid.Retriever.html#start_link/1"},{"type":"module","title":"ExMemvid.TextChunking","doc":"Contains functions for chunking text into smaller segments","ref":"ExMemvid.TextChunking.html"},{"type":"function","title":"ExMemvid.TextChunking.chunk/2","doc":"","ref":"ExMemvid.TextChunking.html#chunk/2"},{"type":"extras","title":"Introduction","doc":"# ExMemvid\n\nAn Elixir Port of [Memvid](https://github.com/Olow304/memvid).\n\nExMemvid is a proof-of-concept library for storing and retrieving large amounts of text data by encoding it into a video file composed of QR code frames. It leverages modern Elixir libraries for machine learning, video processing, and vector search to provide a unique solution for data storage and semantic retrieval.","ref":"readme.html"},{"type":"extras","title":"How it Works - Introduction","doc":"The core idea is to treat video frames as a data storage medium. Each frame in the video contains a QR code that holds a chunk of text. A separate search index is created using text embeddings to allow for fast, semantic searching of the content stored in the video.","ref":"readme.html#how-it-works"},{"type":"extras","title":"Encoding Process - Introduction","doc":"1.  **Text Chunking**: The input text is divided into smaller, manageable chunks.\n2.  **Embedding**: A sentence transformer model from Hugging Face (via `Bumblebee`) generates a vector embedding for each text chunk.\n3.  **QR Code Generation**: Each text chunk is serialized (optionally with Gzip compression) and encoded into a QR code image.\n4.  **Video Encoding**: The QR code images are compiled into a video file, where each image becomes a single frame. The library uses `Xav` and `Evision` (OpenCV bindings) for this.\n5.  **Index Creation**: The vector embeddings are stored in an HNSWLib (Hierarchical Navigable Small World) index for efficient similarity search. This index maps the embeddings to their corresponding frame numbers in the video.\n6.  **Saving**: The final video file and the search index are saved to disk.","ref":"readme.html#encoding-process"},{"type":"extras","title":"Retrieval Process - Introduction","doc":"1.  **Search Query**: The user provides a text query.\n2.  **Query Embedding**: The query is converted into a vector embedding using the same model as the encoding process.\n3.  **Semantic Search**: The HNSWLib index is queried to find the text chunks with embeddings most similar to the query's embedding.\n4.  **Frame Identification**: The search results from the index provide the frame numbers where the relevant text chunks are stored.\n5.  **Frame Decoding**: The `Retriever` seeks to the specific frames in the video file, reads the QR codes, and decodes them to retrieve the original text chunks.\n6.  **Result Aggregation**: The retrieved text chunks are returned to the user.","ref":"readme.html#retrieval-process"},{"type":"extras","title":"Features - Introduction","doc":"*   **Data Archiving**: Store large text corpora in a compressed video format.\n*   **Semantic Search**: Go beyond keyword matching with state-of-the-art text embeddings.\n*   **Configurable**: Easily configure everything from the video codec and QR code version to the embedding model.\n*   **Concurrent**: Utilizes Elixir's concurrency to parallelize embedding and frame decoding tasks.\n*   **Extensible**: The `Embedding` behaviour allows for swapping out the embedding implementation.","ref":"readme.html#features"},{"type":"extras","title":"Installation - Introduction","doc":"Add `ex_memvid` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_memvid, github: \"edlontech/ex_memvid\", branch: \"main\"}\n  ]\nend\n```\n\nYou will also need `ffmpeg` installed on your system for some of the underlying video operations.","ref":"readme.html#installation"},{"type":"extras","title":"Usage Example - Introduction","doc":"Here’s a complete example of how to encode a list of text chunks and then retrieve them via search.\n\n```elixir\n# 1. Configuration\nconfig = ExMemvid.Config.validate!(\n  embedding: %{\n    # Using a mock embedding module for this example\n    module: ExMemvid.MockEmbedding,\n    dimension: 3\n  }\n)\n\n# Some text data to store\nchunks = [\n  \"elixir is a functional, concurrent language\",\n  \"phoenix is a popular web framework for elixir\",\n  \"the actor model enables robust, concurrent systems\",\n  \"bumblebee brings hugging face transformers to elixir\"\n]\n\n# 2. Setup Encoder\n{:ok, encoder} = ExMemvid.Encoder.new(config)\nencoder = ExMemvid.Encoder.add_chunks(encoder, chunks)\n\n# 3. Define output paths\noutput_dir = \"output\"\nvideo_path = Path.join(output_dir, \"my_video.mkv\")\nindex_path = Path.join(output_dir, \"my_index.json\")\n\n# 4. Build the video and search index\n{:ok, stats} = ExMemvid.Encoder.build_video(encoder, video_path, index_path)\nIO.inspect(stats)\n\n# 5. Start the Retriever GenServer\n{:ok, retriever_pid} = ExMemvid.Retriever.start_link(\n  video_path: video_path,\n  index_path: index_path,\n  config: config\n)\n\n# 6. Perform a search\nquery = \"What is elixir?\"\n{:ok, results} = ExMemvid.Retriever.search(retriever_pid, query, 2)\n\nIO.puts(\"Search results for '#{query}':\")\nIO.inspect(results)\n#=> [\"elixir is a functional, concurrent language\", \"bumblebee brings hugging face transformers to elixir\"]\n```","ref":"readme.html#usage-example"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n## [0.1.1](https://github.com/edlontech/ex_memvid/compare/v0.1.0...v0.1.1) (2025-06-24)","ref":"changelog.html"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixing EXLA mess ([edef7e8](https://github.com/edlontech/ex_memvid/commit/edef7e802a9f2588dd33285e99e871da888b7b11))","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"0.1.0 (2025-06-24) - Changelog","doc":"","ref":"changelog.html#0-1-0-2025-06-24"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fixed some tests and added workflow ([6201e25](https://github.com/edlontech/ex_memvid/commit/6201e25e600e985d7ed8d5e906ae4a6520a04a85))","ref":"changelog.html#bug-fixes-1"},{"type":"extras","title":"Continuous Integration - Changelog","doc":"* Improved workflows ([0771a8f](https://github.com/edlontech/ex_memvid/commit/0771a8f0f482f040526faaaf9806de03ec9a19e2))","ref":"changelog.html#continuous-integration"},{"type":"extras","title":"License","doc":"Copyright (c) 2025 Ygor Castor https://ygorcastor.github.io\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}